# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


class AutoMLErrorStrings:
    """
    All un-formatted error strings that accompany the common error codes in AutoML.

    Dev note: Please keep this list sorted on keys.
    """

    # region UserErrorStrings
    ALLOWED_MODELS_NON_EXPLAINABLE = "All models in the allowlist are not explainable, " \
                                     "but model_explainability is set to True. " \
                                     "Please allow one of explainable model or set model_explainability to False. " \
                                     "Non explainable models: {non_explainable_models}."
    ALLOWED_MODELS_SUBSET_OF_BLOCKED_MODELS = "All allowed models are within blocked models list. Please " \
                                              "remove models from the exclude list or add models to the allow list."
    ALLOWED_MODELS_UNSUPPORTED = "Allowed models [{allowed_models}] are not supported for scenario: " \
                                 "{scenario}."
    ALL_ALGORITHMS_ARE_BLOCKED = "All models are blocked. Please ensure that at least one model is allowed."
    ALL_FEATURES_ARE_EXCLUDED = "X should contain at least one feature with a minimum of two unique values to train."
    ALL_TARGETS_NAN = "All the values in the target column are either empty or NaN. Please make sure there are at " \
                      "least two distinct values in the target column."
    ALL_TARGETS_OVERLAPPING = "At least two distinct values for the target column are required for " \
                              "{task_type} task. Please check the values in the target column."
    ALL_TARGETS_UNIQUE = "For a {task_type} task, the label cannot be unique for every sample."
    ARIMAX_EMPTY_DATA_FRAME = "The input data frame cannot be empty."
    ARIMAX_EXTENSION_DATA_MISSING_COLUMNS = "Extension data for series id {tsid} is missing " \
                                            "required columns {column_names}."
    ARIMAX_OLS_FIT_EXCEPTION = "Unable to fit the ARIMAX model, because of exception " \
                               "during fitting of the linear model. " \
                               "Exception: {exception}. Please check your data and run the AutoML again."
    ARIMAX_OLS_LINALG_ERROR = "Unable to fit the ARIMAX model, because " \
                              "linear model does not converge. Please check your data. " \
                              "Original Error: {error}"
    ARIMAX_PDQ_ERROR = "Unable to fit the ARIMAX model, because of exception " \
                       "during estimation of the best hyperparameters."
    ARIMAX_SARIMAX_ERROR = "Unable to fit the ARIMAX model, because of exception " \
                           "during fitting of SARIMAX."
    ARIMA_BAD_MAX_HORIZON = "Forecast horizon must be a positive integer."
    BAD_DATA_IN_WEIGHT_COLUMN = "Weight column contains invalid values such as 'NaN' or 'infinite'"
    CACHE_OPERATION = "Cache operation ('{operation_name}') failed due to an OS error. This is most commonly caused " \
                      "due to insufficient resources on the OS (e.g. process limits, disk space) or inadequate " \
                      "permissions on the project directory ({path}). Error details: {os_error_details}."
    CANCEL_UNSUPPORTED_FOR_LOCAL_RUNS = "Cancel operation is not supported for local runs. Local runs may be " \
                                        "canceled by raising a keyboard interrupt (e.g. via. `Ctrl + C`)"
    COMPUTE_NOT_READY = "Compute not in 'Succeeded' state. Please choose another compute or wait till it is ready."
    CONFLICTING_FEATURIZATION_CONFIG_DROPPED_COLUMNS = "Featurization '{sub_config_name}' customization contains " \
                                                       "columns ({dropped_columns}), which are also configured to be" \
                                                       " dropped via drop_columns. Please resolve the inconsistency " \
                                                       "by reconfiguring the columns that are to be customized."
    CONFLICTING_FEATURIZATION_CONFIG_RESERVED_COLUMNS = "Featurization '{sub_config_name}' customization contains " \
                                                        "reserved columns ({reserved_columns}). Please resolve the " \
                                                        "inconsistency by reconfiguring the columns that are to be " \
                                                        "customized."
    CONFLICTING_TIMEOUT_IN_ARGUMENTS = "run_invocation_timeout (in seconds) should be greater than "\
                                       "experiment_timeout_hours. The run_invocation_timeout (in seconds) "\
                                       "should be set to maximum training time of one AutoML run with a buffer "\
                                       "of at least 300 seconds."
    CONFLICTING_VALUE_FOR_ARGUMENTS = "Conflicting or duplicate values are provided for arguments: [{arguments}]"
    CONTENT_MODIFIED = "The data was modified while being read. Error: {dprep_error}"
    CONTINUE_RUN_UNSUPPORTED_FOR_ADB = "The module '{module_name}' must be installed in order to continue an ADB " \
                                       "experiment. Please install this dependency to allow continuing an " \
                                       "experiment. This can be done via. `pip install {module_name}`."
    CONTINUE_RUN_UNSUPPORTED_FOR_IMAGE_RUNS = "Continuing an AutoML Image run is not supported."
    CONTINUE_RUN_UNSUPPORTED_FOR_UNTRACKED_RUNS = "Continuing this experiment is not supported. This is because " \
                                                  "tracking child runs is disabled for this experiment."
    CREDENTIALLESS_DATASTORE_NOT_SUPPORTED = "Credentialless datastore options are not supported by AutoML. " \
                                             "Please ensure {data_store} has valid credentials associated to " \
                                             "use AutoML."
    DATABASE_QUERY = "Database query error while fetching data. Error: {dprep_error}"
    DATAPREP_SCRIPT_EXECUTION = "Encountered error while fetching data. Error: {dprep_error}"
    DATAPREP_STEP_TRANSLATION = "Encountered step translation error while fetching data. Error: {dprep_error}"
    DATAPREP_VALIDATION = "Validation error while fetching data. Error: {dprep_error}"
    DATASETS_FEATURE_COUNT_MISMATCH = "The number of features in [{first_dataset_name}]({first_dataset_shape}) does " \
                                      "not match with those in [{second_dataset_name}]({second_dataset_shape}). " \
                                      "Please inspect your data, and make sure that features are aligned in " \
                                      "both the Datasets."
    DATASET_CONTAINS_INF = "The input dataset {data_object_name} contains infinity values (e.g. np.inf). " \
                           "Please drop these rows before submitting the experiment again."
    DATASET_CONTAINS_NAN = "The input dataset {data_object_name} contains NaN (np.NaN) values. " \
                           "Please drop these rows before submitting the experiment again or turn on featurization."
    DATASET_FILE_READ = "Fetching data from the underlying storage account timed out. Please retry again after " \
                        "some time, or optimize your blob s񖒈orage performance."
    DATASET_USER_ERROR = "Encountered user error while fetching data from Dataset. Error: {dataset_error}"
    DATASTORE_NOT_FOUND = "The provided Datastore was not found. Error: {dprep_error}"
    DATA_AUTH_BAD = "Datastore {data_store_name} was not accessible. Please update the authentication to the " \
                    "datastore and retry."
    DATA_CONTAIN_ORIGIN = "MaxHorizonFeaturizer cannot featurize data that contains origin times. " \
                          "Input data already has time-series features; " \
                          "please supply a dataset that has not been pre-processed."
    DATA_FROM_MULTIPLE_GROUPS = "The input file {file_name} in the dataset contains data for more than one training " \
                                "level node. Please check the following table to remove those rows\n" \
                                "{unique_df_summary}"
    DATA_MEMORY_ERROR = "Failed to retrieve data from {data} due to MemoryError."
    DATA_NOT_PARTITIONED = "The dataset provided was not partitioned."
    DATA_OTHER_NON_AUTH_ERROR = "Datastore {data_store_name} access failed with error: {exception}."
    DATA_PATH_INACCESSIBLE = "The provided path to the data in the Datastore was inaccessible. Please make sure " \
                             "you have the necessary access rights on the resource. Error: {dprep_error}"
    DATA_PATH_NOT_FOUND = "The provided path to the data in the Datastore does not exist. Error: {dprep_error}"
    DATA_SCRIPT_NOT_FOUND = "The data script (containing get_data function) could not be located at: {script_path}. " \
                            "Ensure that the file exists, and the function get_data (returning X, y, X_valid, " \
                            "y_valid) is callable.\n Please note that this scenario is deprecated. Instead, " \
                            "pass in a training_data and/or validation_data along with a label_column_name as part " \
                            "of AutoMLConfig."
    DATA_SHAPE_MISMATCH = "One or more data input arrays has incompatible dimensions. When providing X, y, weights " \
                          "as separate input arrays, ensure that X is a two-dimensional array, while y and weights " \
                          "are one-dimensional arrays. Note that this way of providing inputs is deprecated, " \
                          "instead use 'training_data' and 'validation_data' to specify data inputs, along with " \
                          "'label' and 'weights' column names."
    DATE_OUT_OF_RANGE_DURING_PADDING = \
        "The padding of time series has failed " \
        "because given the forecast frequency {freq} and the start date {start} it should " \
        "start at the date earlier than {min}."
    DATE_OUT_OF_RANGE_DURING_PADDING_GRAIN = \
        "The padding of short time series with identifier(s) {grain} has failed " \
        "because given the forecast frequency {freq} and the start date {start} it should " \
        "start at the date earlier than {min}."
    DEPENDENCY_WRONG_VERSION = "The model you attempted to retrieve requires '{module}' to be installed at " \
                               "'{ver}'. You have '{module}=={cur_version}', please reinstall '{module}{ver}' " \
                               "(e.g. `pip install {module}{ver}`) and rerun the previous command."
    DISK_FULL = "Operation {operation_name} failed due to low disk space. Please free up some space before " \
                "continuing with the experiment."
    DNN_NOT_SUPPORTED_FOR_MANY_MODEL = "Forecast DNN is not supported for many model solution. " \
                                       "Please disable DNN when running many model solution."
    DUPLICATE_COLUMNS = "'{data_object_name}' consists of duplicate columns: [{duplicate_columns}]. Please either " \
                        "drop the columns that are repeated, or rename one of the columns so that they are unique."
    EMPTY_LAGS_FOR_COLUMNS = "The lags for all columns are represented by empty lists. Please set the " \
                             "target_lags parameter to None to turn off the lag feature and run the experiment again."
    EXECUTION_FAILURE = "Failed to execute the requested operation: {operation_name:log_safe}. Error details: " \
                        "{error_details}"
    EXPERIMENT_TIMED_OUT = "Experiment timeout reached, skipping execution of the child run and terminating the " \
                           "parent run. Consider increasing experiment_timeout_hours."
    EXPERIMENT_TIMEOUT_FOR_DATA_SIZE = "The ExperimentTimeout should be set more than {minimum} minutes with an " \
                                       "input data of rows*cols({rows}*{columns}={total}), and up to {maximum}."
    EXPLAINABILITY_PACKAGE_MISSING = "Cannot complete the model explainability operation due to missing packages: " \
                                     "[{missing_packages}]"
    FEATURE_TYPE_UNSUPPORTED = "The column '{column_name}' in the input Dataset is of unsupported type: " \
                               "'{column_type}'. Supported type(s): '{supported_types}'"
    FEATURE_UNSUPPORTED_FOR_INCOMPATIBLE_ARGUMENTS = "Feature [{feature_name}] is unsupported due to incompatible " \
                                                     "values for argument(s): [{arguments}]"
    FEATURIZATION_CONFIG_COLUMN_MISSING = "The column(s) '{columns}' specified in the Featurization " \
                                          "{sub_config_name} customization is not present in the dataframe. " \
                                          "Valid columns: {all_columns}"
    FEATURIZATION_CONFIG_EMPTY_FILL_VALUE = "A fill value is required for constant value imputation. Please provide " \
                                            "a non-empty value for '{argument_name}' parameter. Example code: " \
                                            "`featurization_config.add_transformer_params('Imputer', ['column_name']" \
                                            ", \"{{'strategy': 'constant', 'fill_value': 0}}\")`"
    FEATURIZATION_CONFIG_FORECASTING_STRATEGY = "Only the following strategies are enabled for a " \
                                                "Forecasting task's target column: ({strategies}). Please fix your " \
                                                "featurization configuration and try again."
    FEATURIZATION_CONFIG_INVALID_TYPE = "The column {column_name} should be able to convert to {data_type} if " \
                                        "featurization configuration's {featurization_option} is enabled. " \
                                        "Please fix your data and try again."
    FEATURIZATION_CONFIG_MULTIPLE_IMPUTERS = "Only one imputation method may be defined for each column. In the " \
                                             "provided configuration, multiple imputers are assigned to the " \
                                             "following columns {columns}\n. Please verify that only one imputer " \
                                             "is defined per column."
    FEATURIZATION_CONFIG_PARAM_OVERRIDDEN = "Failed while {stage} learned transformations. This could be caused by " \
                                            "transformer parameters being overridden. Please check logs for " \
                                            "detailed error messages."
    FEATURIZATION_CONFIG_WRONG_IMPUTATION_VALUE = "A fill value can not be np.inf or np.NaN. Please correct " \
                                                  "the value and run AutoML again."
    FEATURIZATION_REQUIRED = "The training data contains features of type {feature_types}, that can't automatically " \
                             "be processed. Please turn on featurization by setting it as 'auto' or giving " \
                             "custom featurization settings. Feature(s) that require featurization: [{features}]"
    FORECAST_DISTRIBUTED_WRONG_INDICATOR_TYPE = "Wrong data type in the indicator type dictionary was encountered."
    FORECAST_EMPTY_AGGREGATION = "The aggregation of the data set has resulted in empty dataframe. Please use the " \
                                 "dataset with the later dates."
    FORECAST_FIT_NOT_CALLED = "fit() must be called before transform()."
    FORECAST_FIT_NOT_SUPPORT = "For a forecasting model, `fit` is not supported. " \
                               "Please submit an AutoML run to get the fitted model instead."
    FORECAST_HORIZON_EXCEEDED = \
        "Input prediction data X_pred or input forecast_destination contains dates later than " \
        "the forecast horizon. Please shorten the prediction data so that it is within the " \
        "forecast horizon or adjust the forecast_destination date."
    FORECAST_MISALIGNED_TIMES = \
        "Input prediction data contains times which are not aligned with timeseries frequency " \
        "determined during training. Please check the prediction data frequency and whether it forms " \
        "a regularly spaced grid with the training data times. " \
        "If the misalignment is expected, please use the `ignore_data_errors=True` option in forecast() " \
        "or rolling_forecast(). In this case, the misaligned rows will be not be forecasted. " \
        "The forecaster will impute values at the missing grid point in this case as well, so treat results " \
        "with caution."
    FORECAST_PREDICT_NOT_SUPPORT = "For a forecasting model, `predict` is not supported. " \
                                   "Please use `forecast` instead."
    FORECAST_SERIES_NOT_IN_TRAIN = "No prediction available for series id {tsid} because " \
                                   "it was not included in training data."
    GENERIC_FIT_EXCEPTION = "Failed to fit the input data using {transformer_name}"
    GENERIC_PREDICT_EXCEPTION = "Failed to predict from the input data using {transformer_name}"
    GENERIC_TRANSFORM_EXCEPTION = "Failed to transform the input data using {transformer_name}"
    GRAIN_ABSENT = "The time series identifier {grain} does not exist in the training dataset. " \
                   "Please inspect your data."
    GRAIN_COLUMNS_AND_GRAIN_NAME_MISMATCH = "The number of time series identifier columns is different " \
                                            "from the length of a time series identifier name."
    GRAIN_SHORTER_THAN_TEST_SIZE = "Some time series identifiers in the input dataset are shorter than the " \
                                   "test size. For a test_size of {test_size}, " \
                                   "the minimum number of rows per time series identifier must be at least " \
                                   "{min_rows_per_grain}. Please either try to reduce the test_size or provide more " \
                                   "data per time series identifier."
    HIERARCHY_ALL_PARALLEL_FAILURE = "All the steps in {parallel_step} failed with UserError. " \
                                     "Please check {file_name} in artifacts for details."
    HIERARCHY_EXPLANATIONS_NOT_FOUND = "The {exp_type} explanations were not found."
    HIERARCHY_NO_TRAINING_RUN = "No valid training run can be found in the experiment. Please make sure" \
                                " that at least one successful training run is in the same experiment."
    HIERARCHY_PREDICTIONS_NOT_FOUND = "Unable to allocate predictions as no intermediate model predictions were found."
    HTTP_CONNECTION_FAILURE = "Failed to establish HTTP connection to the service. This may be caused due the local " \
                              "compute being overwhelmed with HTTP requests. Please make sure that there are " \
                              "enough network resources available for the experiment to run. More details: " \
                              "{error_details}"
    IMAGE_DUPLICATE_PARAMETERS = "Found duplicate entries for AutoML Image parameter '{parameter_name}'. " \
                                 "Please provide only one entry."
    IMAGE_ODD_NUM_ARGUMENTS = "An even number of elements in the arguments list is required. " \
                              "[{num_arguments}] elements were provided."
    IMAGE_PARAMETER_SPACE_EMPTY = "The parameter space must contain at least one parameter."
    INCOMPATIBLE_DEPENDENCY = "The installed version of '{package}' does not match the expected version.\n" \
                              "Installed: {current_package}\nExpected: {expected_package}\n" \
                              "Install the required package by running 'pip install \"{expected_package}\"'."
    INCOMPATIBLE_OR_MISSING_DEPENDENCY = "Install the required versions of packages using the requirements file. " \
                                         "Requirements file location: {validated_requirements_file_path}. " \
                                         "Alternatively, use remote target to avoid dependency management. \n" \
                                         "{missing_packages_message_header}\n{missing_packages_message}"
    INCOMPATIBLE_OR_MISSING_DEPENDENCY_DATABRICKS = "Install the required versions of packages by setting up the " \
                                                    "environment as described in documentation.\n" \
                                                    "{missing_packages_message_header}\n{missing_packages_message}"
    INCONSISTENT_COLUMN_TYPE_IN_TRAIN_VALID = "Datatype for column {column_name} was detected as {train_dtype} in " \
                                              "the training set, but was found to be {validation_dtype} in the " \
                                              "validation set. Please ensure that the datatypes between training " \
                                              "and validation sets are aligned."
    INCONSISTENT_NUMBER_OF_SAMPLES = "The number of samples in {data_one} and {data_two} are inconsistent. If you " \
                                     "are using an AzureML Dataset as input, this may be caused as a result of " \
                                     "having multi-line strings in the data. Please make sure that " \
                                     "'support_multi_line' is set to True when creating a Dataset. Example: " \
                                     "Dataset.Tabular.from_delimited_files('http://path/to/csv', " \
                                     "support_multi_line = True)"
    INDISTINCT_LABEL_COLUMN = "The label column {label_column_name} was also contained within the training data. " \
                              "Please rename this column in either X or y."
    INPUT_DATASET_EMPTY = "The provided Dataset contained no data. Please make sure there are non-zero number " \
                          "of samples and features in the data."
    INPUT_DATA_WITH_MIXED_TYPE = "A mix of Dataset and Pandas objects provided. " \
                                 "Please provide either all Dataset or all Pandas objects."
    INSUFFICIENT_GPU_MEMORY = "There is not enough GPU memory on the machine to do the operation. " \
                              "Please try running the experiment on a VM with higher GPU memory, decrease the batch " \
                              "size or image size."
    INSUFFICIENT_MEMORY = "There is not enough memory on the machine to do the operation. " \
                          "Please try running the experiment on a VM with higher memory."
    INSUFFICIENT_MEMORY_LIKELY = "'Subprocess (pid {pid}) killed by unhandled signal {errorcode} ({errorname}). " \
                                 "This is most likely due to an out of memory condition. " \
                                 "Please try running the experiment on a VM with higher memory."
    INSUFFICIENT_MEMORY_WITH_HEURISTICS = "There is not enough memory on the machine to do the operation. " \
                                          "The amount of available memory is {avail_mem} out of {total_mem} total " \
                                          "memory. To fit the model at least {min_mem} more memory is required. " \
                                          "Please try running the experiment on a VM with higher memory.\n" \
                                          "If your data have multiple time series, you can also consider using many " \
                                          "model solution to workaround the memory issue, please refer to " \
                                          "https://aka.ms/manymodel for details."
    INSUFFICIENT_SAMPLES_FOR_MINORITY_CLASS = "Number of samples for the class '{class_label}' in training dataset " \
                                              "is fewer ({count:log_safe}) than the number of cross-validation " \
                                              "folds ({n_cv:log_safe}) requested. Consider a more balanced dataset " \
                                              "with sufficient number of samples for the minority class, or reduce " \
                                              "the number of CV folds such that each fold can be generated with a " \
                                              "distribution similar to the overall training dataset."
    INSUFFICIENT_SAMPLE_SIZE = "The training dataset '{data_object_name}' has {sample_count} usable records (i.e. " \
                               "those which are not NaN or None). That is less than the minimum required training " \
                               "sample size of {minimum_count}. Consider adding more data points to ensure better " \
                               "model accuracy."
    INSUFFICIENT_SHM_MEMORY = "There is not enough shared memory on the machine to do the operation. " \
                              "Please try running the experiment with a smaller batch size."
    INVALID_ARGUMENT_FOR_TASK = "Invalid argument(s) '{arguments}' for task type '{task_type}'."
    INVALID_ARGUMENT_TYPE = "Argument [{argument}] is of unsupported type: [{actual_type}]. " \
                            "Supported type(s): [{expected_types}]"
    INVALID_ARGUMENT_TYPE_WITH_CONDITION = "Argument [{argument}] is of unsupported type: [{actual_type}] when " \
                                           "'{condition_str}'.\n" \
                                           "Supported type(s): [{expected_types}]"
    INVALID_ARGUMENT_WITH_SUPPORTED_VALUES = "Invalid argument(s) '{arguments}' specified. " \
                                             "Supported value(s): '{supported_values}'."
    INVALID_ARGUMENT_WITH_SUPPORTED_VALUES_FOR_TASK = "Invalid argument(s) '{arguments}' specified for task type " \
                                                      "'{task_type}'. Supported value(s): '{supported_values}'."
    INVALID_COMPUTE_TARGET_FOR_DATABRICKS = "Databricks compute cannot be directly attached for AutoML runs. " \
                                            "Please pass in a spark context instead using the spark_context " \
                                            "parameter and set compute_target to 'local'."
    INVALID_CV_SPLITS = "cv_splits_indices should be a List of List[numpy.ndarray]. Each List[numpy.ndarray] " \
                        "corresponds to a CV fold and should have just 2 elements: The indices for training set " \
                        "and for the validation set."
    INVALID_DAMPING_SETTINGS = "Conflicting values are provided for arguments [{model_type}] and [{is_damped}]. " \
                               "Damping can only be applied when there is a trend term."
    INVALID_FEATURIZER = "[{featurizer_name}] is not a valid featurizer for featurizer type: [{featurizer_type}]"
    INVALID_FORECAST_DATE = "The forecast date ({forecast_date}) " \
                            "cannot be less than the first observed date in the training data " \
                            "({first_observed_date}). Please inspect the data."
    INVALID_FORECAST_DATE_FOR_GRAIN = "The forecast date ({forecast_date}) for time series identifier '{grain}' " \
                                      "cannot be less than the first observed date in the training data " \
                                      "({first_observed_date}). Please inspect the data."
    INVALID_INPUT_DATATYPE = "Input of type '{input_type}' is not supported. Supported types: [{supported_types}]" \
                             "Please refer to documentation for converting to Supported types: " \
                             "https://docs.microsoft.com/en-us/python/api/azureml-core/azureml.core.dataset.dataset" \
                             "?view=azure-ml-py"
    INVALID_ITERATION = "Invalid iteration. Run {run_id} has {num_iterations} iterations."
    INVALID_LABEL_COLUMN_VALUES = "Found one or more of the following issues with the label column. " \
                                  "1) There more than than 2,147,483,647 labels. " \
                                  "2) Label column type is not valid for multi-class. " \
                                  "3) Label column contains invalid values for multi-class."
    INVALID_METRIC_FOR_SINGLE_VALUED_COLUMN = "The data in {data_object_name} is single valued. Please make sure " \
                                              "that the data is well represented with all classes for a " \
                                              "classification task. Or please try one of the following as primary " \
                                              "metrics: {valid_primary_metrics}."
    INVALID_ONNX_DATA = "The onnx resource passed in is invalid. Please make sure the x_train data is a pandas " \
                        "DataFrame and it has column names during training."
    INVALID_OPERATION_ON_RUN_STATE = "Operation [{operation_name}] on the RunID [{run_id}] is invalid. " \
                                     "Current run state: [{run_state}]"
    INVALID_PARAMETER_SELECTION = "The {parameter} parameter may take value of {values}."
    INVALID_SERIES_FOR_STL = "Cannot calculate STL decomposition on a series with NaN values. Please either fill in " \
                             "these values or drop these rows from the series."
    INVALID_STL_FEATURIZER_FOR_MULTIPLICATIVE_MODEL = "Cannot use multiplicative model type [{model_type}] because " \
                                                      "trend contains negative or zero values."
    INVALID_VALUES_IN_CV_SPLIT_COLUMN = "The CV split column must contain integer values from the set [0, 1], and " \
                                        "not all same such that it divides the dataset into a train-valid split. A " \
                                        "value of 1 indicates the row should be used for training while 0 indicates " \
                                        "the row should be used for validation."
    INVALID_VALUES_IN_DATA = "The input dataset cannot be pre-processed. This is usually caused by having an " \
                             "invalid value in one of the cells (e.g. a numpy array), or the rows in the dataset " \
                             "having different shapes. Clean up the data manually before re-submitting."
    ITERATION_TIMED_OUT = "Iteration timeout reached, skipping execution of the child run. " \
                          "Consider increasing iteration_timeout_minutes."
    LARGE_DATA_ALGORITHMS_UNSUPPORTED = "AveragedPerceptronClassifier, FastLinearRegressor, " \
                                        "OnlineGradientDescentRegressor are supported only in AutoML SDK " \
                                        "versions 1.48.0 or below. Consider upgrading to AutoML SDK V2 to evaluate " \
                                        "distributed training algorithms that can handle large data."
    LOAD_MODEL_DEPENDENCY_MISSING = "The model you attempted to retrieve requires '{module}' to be installed at " \
                                    "'{ver}'. Please install '{module}{ver}' (e.g. `pip install {module}{ver}`) and " \
                                    "then rerun the previous command."
    LOCAL_INFERENCE_UNSUPPORTED = "Local inference is not supported for this feature."
    LOCAL_MANAGED_USER_ERROR = "Local managed run failed when trying to submit to the user compute with " \
                               "error type: {error_type}"
    MALFORMED_JSON_STRING = "Failed to parse the provided JSON string. Error: {json_decode_error}"
    MEMORY_EXHAUSTED = "There is not enough memory on the machine to fit the model. " \
                       "The amount of available memory is {avail_mem} out of {total_mem} " \
                       "total memory. To fit the model at least {min_mem} more memory is required. " \
                       "Please install more memory or use bigger virtual " \
                       "machine to generate model on this dataset."
    METHOD_NOT_FOUND = "Required method [{method_name}] is not found."
    MISSING_CACHE_CONTENTS = "Some of the intermediate data that was cached on the local disk (at path {path}) are " \
                             "no longer available. Missing file(s): {files}. Please try submitting the experiment " \
                             "again."
    MISSING_COLUMNS_IN_DATA = "Expected column(s) {columns} not found in {data_object_name}."
    MISSING_CREDENTIALS_FOR_WORKSPACE_BLOB_STORE = "Could not create a connection to the AzureFileService due to " \
                                                   "missing credentials. Either an Account Key or SAS token needs " \
                                                   "to be linked the default workspace blob store."
    MISSING_POSITIVE_LABEL = "Positive class label {positive_label} is missing from label column. " \
                             "Please check your positive_label and try again."
    MISSING_SECRETS = "Failed to get data from the Datastore due to missing secrets. Error: {dprep_error}"
    MISSING_VALIDATION_CONFIG = "No form of validation was provided. Please provide either a validation dataset, or " \
                                "specify the type of validation you would like to use."
    MODEL_EXPLANATIONS_DATA_METADATA_DIMENSION_MISMATCH = "Mismatch found between number of features " \
                                                          "{number_of_features} and number of feature names " \
                                                          "{number_of_feature_names}"
    MODEL_EXPLANATIONS_FEATURE_NAME_LENGTH_MISMATCH = "Mismatch found between number of feature names " \
                                                      "{number_of_feature_names} and a dimension of the " \
                                                      "feature map {dimension_of_feature_map}"
    MODEL_EXPLANATIONS_UNSUPPORTED_FOR_ALGORITHM = "Model explanations are currently unsupported for {algorithm_name}."
    MODEL_MISSING = "Could not find a model with valid score for metric '{metric}'. Please ensure that at least one " \
                    "run was successfully completed with a valid score for the given metric."
    MODEL_NOT_FOUND = "Model with name {model_name} was not found in the workspace. Either this group was " \
                      "not seen during training or this group failed at training time. " \
                      "Please make sure this group exists in training data and had successful training."
    MODEL_NOT_PICKLABLE = "Model with name {model_name} was not deserializable in the current environment. " \
                          "Please use this environment to train the model."
    MODEL_NOT_SUPPORTED = "The model {model_name} is not supported by current " \
                          "version of AutoML and will not be used. " \
                          "Please upgrade SDK to the latest version to enable it."
    NETWORK_VNET_MISCONFIG = "A virtual network '{vnet}' appears to have been set on '{cluster_name}' compute. " \
                             "Please verify that the required inbound and outbound network traffic permissions are " \
                             "configured in the NSG and firewall of the VNet. For more details, refer to " \
                             "https://learn.microsoft.com/en-us/azure/machine-learning/how-to-access-azureml" \
                             "-behind-firewall?tabs=ipaddress%2Cpublic ."
    NLP_ARGUMENT_PARSING_ERROR = "Encountered exception while parsing command-line arguments: {details}"
    NLP_CLASSIFICATION_INSUFFICIENT_COLUMNS = "Multi-class and multi-label classification training requires at" \
                                              " least one non-label column."
    NLP_CLASSIFICATION_INSUFFICIENT_EXAMPLES = "Multi-class and multi-label classification training requires at" \
                                               " least {exp_cnt:log_safe} samples (with non-null labels) for" \
                                               " {split_type:log_safe} set, but only {act_cnt:log_safe} were found."
    NLP_COLUMN_ORDERING_MISMATCH = "Columns in training set and validation set should follow the same order. " \
                                   "First inconsistency observed at index {bad_idx:log_safe}."
    NLP_COLUMN_SET_MISMATCH = "Found {num_cols:log_safe} columns present in either ONLY the training set " \
                              "or the validation set. Train/valid column sets must be identical."
    NLP_COLUMN_TYPE_MISMATCH = "Column has inconsistent types between training and validation sets. " \
                               "See column '{col_name}' at index {bad_idx:log_safe}."
    NLP_CONSECUTIVE_BLANK_LINES = "{split_type:log_safe} set file should not have consecutive blank lines."
    NLP_DUPLICATE_COLUMN_NAMES = "{split_type:log_safe} set should not have duplicated column names."
    NLP_DUPLICATE_LABEL_TYPE_MISMATCH = "Found same label as both string and integer. Please be consistent in " \
                                        "typing of label names."
    NLP_EMPTY_NER_FILE_BEGINNING = "Input NER file should not start with an empty line."
    NLP_EMPTY_NER_FILE_ENDING = "󪟲xactly one empty line is expected at the end of the {split_type:log_safe} set file."
    NLP_EMPTY_NER_TOKEN = "Token should not be empty string. Please follow the NER input guidelines, which can be " \
                          "found at {info_link:log_safe}."
    NLP_INSUFFICIENT_UNIQUE_LABELS = "Training data must contain at least {exp_cnt:log_safe} unique classes, " \
                                     "but only {act_cnt:log_safe} were found."
    NLP_INVALID_NER_DATASET = "Unsupported input data type is used for NER run. Please double check input and try " \
                              "again. NER expects CoNLL formatted text file (.txt) as input. More information can " \
                              "be found at {info_link:log_safe}."
    NLP_INVALID_SWEEP_ARGUMENT = "Invalid sweeping argument detected. {log_safe_description:log_safe}."
    NLP_LABELING_DATA_CONVERSION_FAILED = "Data conversion failed. Make sure the data is of correct format. " \
                                          "Error details: {error_details}"
    NLP_LABELING_DATA_DOWNLOAD_FAILED = "Could not download labeling data. Please check the logs for more details. " \
                                        "Error details: {error_details}"
    NLP_MALFORMED_LABEL_COLUMN = "Failed while parsing input label column in {split_type:log_safe} dataset. " \
                                 "Please make sure the correct format was used, which can be found at " \
                                 "{info_link:log_safe}."
    NLP_MALFORMED_NER_INFERENCE_INPUT = "If a list is provided to the NER model wrapper, it must contain a single " \
                                        "element corresponding to the CoNLL-formatted input string. The provided " \
                                        "list had {list_length:log_safe} elements."
    NLP_MALFORMED_NER_LABELS = "All labels in {split_type:log_safe} set should either start with 'I-' or 'B-' or " \
                               "be exactly 'O'. More information about NER input formats can be found at " \
                               "{info_link:log_safe}."
    NLP_MALFORMED_NER_LINE = "Each line in {split_type:log_safe} set should either contain exactly one white space " \
                             "to separate a token and its label, or it should be an empty line. More information " \
                             "about NER input formats can be found at {info_link:log_safe}."
    NLP_MISSING_DATASET = "{split_type:log_safe} Dataset is either missing or None. Unable to proceed with the " \
                          "training run. More information can be found at {info_link:log_safe}."
    NLP_MISSING_LABEL_COLUMN = "Specified label column '{col_name}' not found in input dataset."
    NLP_MIXED_MULTILABEL_TYPES = "Expected all labels to be of type integer or string, but found {bad_type:log_safe}."
    NLP_NER_INFERENCE_TYPE_MISMATCH = "A CoNLL-formatted input string or a single numpy array containing a " \
                                      "CoNLL-formatted input string are currently the only accepted input types for " \
                                      "NER inferencing. An input of type {bad_type} was encountered instead. Please " \
                                      "update the input and try again."
    NLP_NER_INSUFFICIENT_EXAMPLES = "NER training requires at least {exp_cnt:log_safe} samples for" \
                                    " {split_type:log_safe} set, but only {act_cnt:log_safe} were found."
    NLP_NETWORK_VNET_MISCONFIG = "A virtual network '{vnet}' appears to have been set on '{cluster_name}' compute. " \
                                 "Please verify that the required inbound and outbound network traffic permissions " \
                                 "are configured in the NSG and firewall of the VNet. For more details, refer to " \
                                 "{info_link:log_safe}."
    NLP_UNEXPECTED_LABEL_FORMAT = "Encountered unexpected label format. Please follow the correct formats, " \
                                  "which can be found at {info_link:log_safe}."
    NON_DNN_TEXT_FEATURIZATION_UNSUPPORTED = "For non-English pre-processing of text data, please set " \
                                             "enable_dnn=True and make sure you are using GPU compute."
    NON_OVERLAPPING_COLUMNS_IN_TRAIN_VALID = "Some columns that are present in the validation dataset are absent " \
                                             "from the training dataset. Missing columns: [{missing_columns}]. " \
                                             "Please make sure that the columns in both the datasets are consistent."
    NO_FEATURE_TRANSFORMATIONS_ADDED = "No features could be identified or generated for the given data. " \
                                       "Please pre-process the data manually, or provide custom featurization options."
    NO_METRICS_DATA = "No metrics related data was present at the time of \'{metric}\' calculation either because " \
                      "data was not uploaded in time or because no runs were found in completed state. " \
                      "If the former, please try again in a few minutes."
    NUMBER_COLUMN_CONVERSION_ERROR = "The column {column} cannot be converted to number. Please check your data " \
                                     "and run the AutoML again."
    N_CROSS_VALIDATIONS_EXCEEDS_TRAINING_ROWS = "Number of usable training rows ({training_rows}) (i.e. excluding " \
                                                "rows with NaN or invalid target values)  is less than total " \
                                                "requested CV splits ({n_cross_validations}). " \
                                                "Please reduce the number of splits requested, or increase the size " \
              